import hashlib
import json
from pathlib import Path
from typing import Union


def calculate_sha256(file_path: Union[str, Path]) -> str:
    """
    Calculate the sha256 hash of a file.
    """
    sha256_hash = hashlib.sha256()
    with open(str(file_path), "rb") as f:
        # Read and update hash string value in blocks of 4K
        for byte_block in iter(lambda: f.read(4096), b""):
            sha256_hash.update(byte_block)
    return sha256_hash.hexdigest()


if __name__ == "__main__":
    # 不计算_开头的文件和.py 文件
    data = Path("src/cfundata/data")
    jsonname = "sha256.json"
    use_suffix = [".pt", ".onnx", ".json", ".bin", ".parquet", ".ttc", ".ttf"]
    exclude = [jsonname, "README.md", "LICENSE"]
    # 筛选需要的文件
    files = [f for f in data.rglob("*") if f.is_file() and f.suffix in use_suffix]
    # 排除以. 或者 _ 开头的文件
    files = [f for f in files if not f.name.startswith((".", "_"))]
    # 排除指定文件
    files = [f for f in files if f.name not in exclude]

    result = []
    baseURL = "https://modelscope.cn/datasets/zscmmm/cfundata/resolve/master/"
    for file in files:
        sha256 = calculate_sha256(file)
        name = file.name
        result.append(
            {
                "name": name,
                "url": baseURL + file.parent.name + "/" + name,
                "sha256": sha256,
            }
        )
    # 写为 json 文件
    # print(result)
    json_path = jsonname  # 手动移动到目标目录 data / jsonname
    result = sorted(result, key=lambda x: x["name"])

    with open(str(json_path), "w", encoding="utf-8") as f:
        json.dump(result, f, indent=2, ensure_ascii=False, sort_keys=True)

    print(f"{jsonname} saved to {json_path}")



from dataclasses import asdict
from pathlib import Path

from cfundata import cdata


def test_paths():
    print("Testing paths...")
    print(cdata)
    # 对每个属性检查
    # 每个属性都是一个Path对象
    for name, path in asdict(cdata).items():
        print(f"path: {name} = {path}")
        assert isinstance(path, Path), f"{name} is not a Path object: {path}"
        assert Path(path).exists(), f"Path {name} does not exist: {path}"


if __name__ == "__main__":
    test_paths()
    print("All tests passed!")



"""使用pooch模块来下载和缓存数据文件

优点：
    在本地下载并缓存您的数据文件（因此只需下载一次）
    通过验证加密哈希，确保运行代码的每个人都具有相同版本的数据文件。
    ....

参考：
    [https://www.fatiando.org/pooch/latest/about.html](https://www.fatiando.org/pooch/latest/about.html)
"""

import importlib.resources as pkg_resources
import json
from dataclasses import dataclass
from pathlib import Path

import pooch


def _read_json(path: Path | str) -> list[dict]:
    with path.open("r", encoding="utf-8") as f:
        return json.load(f)


# 5. 构建数据类
@dataclass(frozen=True)
class DataPath:
    """数据路径类，包含该包的所有数据文件的路径

    该类用于存储数据文件的路径，提供了对数据文件的访问方式。
    该类的属性是不可变的，使用时请直接访问属性。
    默认会自动下载到缓存目录中，缓存目录中.由pooch模块管理。

    Attributes:
        DX_DET_ONNX (Path): DX检测模型的ONNX文件路径
        DX_CLS_ONNX (Path): DX分类模型的ONNX文件路径
        DX_DET_PT (Path): DX检测模型的PyTorch文件路径
        DX_CLS_PT (Path): DX分类模型的PyTorch文件路径
        FONT_SIMSUN (Path): SimSun字体文件路径
        FREQUENCY (Path): 频率数据文件路径
        FREQUENCY2 (Path): 频率数据文件路径2
        SFZ_DET_ONNX (Path): sfz检测模型的ONNX文件路径, 大约7000张图片训练的，返回四个类别框，0,3代表反面（国徽），1,2代表正面(头像)

    !!! note
        DX_* 的文件,是dx的点选和语序yzm训练而来的, 大约采用了1.4w张图片训练的.
        模型采用的是yolov11训练而来的, 目标检查和分类模型都含有两种格式的模型, 分别是pt和onnx格式,
        有些时候装ultralytics太慢, 使用onnx模型就快很多。

    !!! note
        SimSun字体文件是用于显示中文的字体文件, 该文件是从Windows系统中提取的, 可能会有版权问题,
        请在使用时注意版权问题。

    频率数据参考: [zoushucai/textfrequency](https://github.com/zoushucai/textfrequency)


    Example:
        ```python
        from cfundata import cdata
        # cdata 是一个DataPath对象, 包含了所有数据文件的路径
        print(cdata.DX_DET_ONNX)
        print(cdata.DX_CLS_ONNX)
        print(cdata)
        ```
    """

    DX_DET_ONNX: Path
    DX_CLS_ONNX: Path
    DX_DET_PT: Path
    DX_CLS_PT: Path
    FONT_SIMSUN: Path
    FREQUENCY: Path
    FREQUENCY2: Path
    SFZ_DET_ONNX: Path
    GT3_WORD_CLS_ONNX: Path
    GT3_WORD_DET_ONNX: Path


def initialize_datapath() -> DataPath:
    """初始化数据路径

    该函数用于初始化数据路径, 包括下载数据文件和缓存数据文件.
    该函数会自动检测数据文件是否存在, 如果不存在, 则会自动下载数据文件.

    Args:
        None

    Returns:
        DataPath: 数据路径对象, 包含所有数据文件的路径.
    """
    try:
        import tqdm

        show_bar = True
    except ImportError:
        show_bar = False

    # 1. 读取资源描述文件
    sha256_path = pkg_resources.files("cfundata.data").joinpath("sha256.json")
    resources = _read_json(sha256_path)

    # 2. 构造 registry 和 urls
    registry = {r["name"]: r["sha256"] for r in resources}
    urls = {r["name"]: r["url"] for r in resources}

    # 3. 初始化 pooch
    pup = pooch.create(
        path=pooch.os_cache("cfundata"),
        base_url="",  # 每个文件有独立 URL
        registry=registry,
        urls=urls,
    )

    # 4. 加载路径：优先从包内加载，失败则从缓存中下载, 且强制返回Path对象
    paths: dict[str, Path] = {}
    for name in registry:
        package_path = pkg_resources.files("cfundata.data").joinpath(name)
        try:
            # 使用 with as_file
            with pkg_resources.as_file(package_path) as local_path:
                p = (
                    local_path
                    if local_path.exists()
                    else pup.fetch(name, progressbar=show_bar)
                )
                paths[name] = Path(p)
        except Exception:
            paths[name] = Path(pup.fetch(name, progressbar=show_bar))

    # 5. 显式验证需要的关键文件存在（防止 KeyError）
    field_map = {
        "DX_DET_ONNX": "dx_det.onnx",
        "DX_CLS_ONNX": "dx_cls.onnx",
        "DX_DET_PT": "dx_det.pt",
        "DX_CLS_PT": "dx_cls.pt",
        "FONT_SIMSUN": "simsun.ttc",
        "FREQUENCY": "frequency.parquet",
        "FREQUENCY2": "frequency2.parquet",
        "SFZ_DET_ONNX": "sfz_det.onnx",
        "GT3_WORD_CLS_ONNX": "gt3_word_cls.onnx",
        "GT3_WORD_DET_ONNX": "gt3_word_det.onnx",
    }
    missing = [fname for fname in field_map.values() if fname not in paths]
    if missing:
        raise ValueError(f"Missing required files: {missing}")

    # 6 自动构建 DataPath 对象
    return DataPath(**{field: paths[filename] for field, filename in field_map.items()})



import os

from modelscope.hub.api import HubApi

api = HubApi()
token = os.getenv("MS_TOKEN")
assert token, "Please set MS_TOKEN in your environment variables."

api.login(token)


owner_name = "zscmmm"
dataset_name = "cfundata"

api.upload_folder(
    repo_id=f"{owner_name}/{dataset_name}",
    folder_path="src/cfundata",
    commit_message="upload dataset folder to repo",
    repo_type="dataset",
    allow_patterns=["data/*"],
    # allow_patterns=["*.ttc", "*.parquet", "*.onnx", "*.pt", "*.json"],
    ignore_patterns=["*/_*", "*.py"],
)

print("Upload completed!")
